-- 24.03.2010 (prot warrior module by Caszum)
--
-- "Caszum" of Kilrogg Protection Warrior experimental module 24.03.10
-- Using the priority system of: "Shield Slam > Revenge > Devastate (Lower than 5 stacks) > Shockwave > Thunderclap > Demoralizing Shout (Debuff off) > Devastate"

do
	local _, playerClass = UnitClass("player")
	if playerClass ~= "WARRIOR" then return end
end

local MODULE = "Prot Warrior"
local Faceroller = _G["Faceroller"]
local gsi = Faceroller.gsi
local w_sund, w_demo, d_demo, l_demo, p_demo
local w_clap, d_clap, f_clap, p_clap


local Init = function(opts)

	options = opts

	if opts.demo == nil then
		opts.demo = true
	end

	if opts.tclap == nil then
		opts.tclap = false
	end

	if opts.shout == nil then
		opts.shout = true
	end

	opts.gui = {
		demo = {
			type = "toggle",
			name = "Suggest Demoralizing Shout",
			get = function ()
				return opts.demo
			end,
			set = function (i, v)
				opts.demo = v
			end,
		},

		tclap = {
			type = "toggle",
			name = "Suggest Thunder Clap when its off cooldown instead of duration is low",
			get = function ()
				return opts.tclap
			end,
			set = function (i, v)
				opts.tclap = v
			end,
		},

		shout = {
			type = "toggle",
			name = "Suggest Commanding/Battle Shout",
			get = function ()
				return opts.shout
			end,
			set = function (i, v)
				opts.shout = v
			end,
		},
	}

	local spells = {
		[30356] = 0,	-- shield slam.
		[46968] = 0,	-- shockwave
		[12809] = 0,	-- concussion blow	
		[57823] = 0,	-- revenge
		[47502] = 0,	-- thunderclap.
		[47437] = 0,	-- demoralizing shout.
		[30022] = 0,	-- devastate
		[47436] = 0,
		[47440] = 0,
	}

	for k, v in pairs(spells) do
		if Faceroller:HasSpell (k) == false then
			return nil
		end
	end

	Faceroller:RegisterBuff(50227)		-- Sword and Board
	Faceroller:RegisterBuff(2565)		-- Shield Block
	Faceroller:RegisterBuff(47440)		-- Commanding Shout
	Faceroller:RegisterBuff(47436)		-- Battle shout
	Faceroller:RegisterDebuff(7386)		-- Sunder armor

	-- Melee Attack Speed Slow Debuff
	Faceroller:RegisterDebuff(47502)	-- Thunder Clap
	Faceroller:RegisterDebuff(49909)	-- Icy Touch
	Faceroller:RegisterDebuff(48485)	-- Infected Wounds
	Faceroller:RegisterDebuff(53696)	-- Judgements of the Just

	-- Attack Power Debuff
	Faceroller:RegisterDebuff(47437)	-- Demoralizing Shout
	Faceroller:RegisterDebuff(48560)	-- Demoralizing Roar
	Faceroller:RegisterDebuff(50511)	-- Curse of Weakness
	Faceroller:RegisterDebuff(26016)	-- Vindication

	w_sund = gsi[7386]
	w_swab = gsi[50227]
	w_blok = gsi[2565]
	w_cout = gsi[47440]
	w_bout = gsi[47436]
	w_clap = gsi[47502]
	d_clap = gsi[49909]
	f_clap = gsi[48485]
	p_clap = gsi[53696]
	w_demo = gsi[47437]
	f_demo = gsi[48560]
	l_demo = gsi[50511]
	p_demo = gsi[26016]
	
	return spells
end

local NextShot = function(gcd, spells, buffs, debuffs, myDebuffs)
	local th = max(1.0, gcd) + 0.1
	local mob = UnitLevel("target");
	
	if buffs[w_blok].active then
		if buffs[w_swab].active or spells[30356].cd < th then
			return 30356 -- Shield Slam
		elseif spells[57823].cd < th and spells[57823].usable then
			return 57823 -- Revenge
		else
			return 47498 -- Devastate
		end
	else
		if buffs[w_swab].active or spells[30356].cd < th then
			return 30356 -- Shield Slam
		elseif spells[57823].cd < th and spells[57823].usable then
			return 57823 -- Revenge
		elseif debuffs[w_sund].active == false or debuffs[w_sund].count ~= 5 or debuffs[w_sund].time_left < 10 then
			return 47498 -- Devastate
		elseif spells[46968].cd < th then
			return 46968 -- Shockwave
		elseif debuffs[w_clap].active == false and debuffs[d_clap].active == false and debuffs[f_clap].active == false and debuffs[p_clap].active == false and spells[47502].cd < th then
			return 47502 -- Thunder Clap
		elseif options.tclap == false and spells[47502].cd < th and debuffs[w_clap].time_left < 10 and debuffs[w_clap].active == true and debuffs[d_clap].active == false and debuffs[f_clap].active == false and debuffs[p_clap].active == false then
			return 47502 -- Thunder Clap
		elseif options.tclap == true and spells[47502].cd < th then
			return 47502 -- Thunder Clap
		elseif options.demo == true and debuffs[f_demo].active == false and debuffs[l_demo].active == false and debuffs[p_demo].active == false and debuffs[w_demo].active == false or options.demo == true and debuffs[w_demo].active == true and debuffs[w_demo].time_left < 10 then
			return 47437 -- Demoralizing Shout
		elseif options.shout == true then
			if buffs[w_cout].time_left < 30 and buffs[w_bout].active == false then
				return 47440 -- Comanding Shout
			elseif buffs[w_bout].time_left < 30 and buffs[w_cout].active == false then
				return 47436 -- Battle Shout
			else 
				return 47498 -- Devastate
			end
		else
			return 47498 -- Devastate
		end
	end
end

local Options = function(cmd)
	if cmd == nil or cmd == "help" then
		print (MODULE .. ": Options:")
		print (MODULE .. ": tclap - Enable/Disable suggesting Thunder Clap when its off cooldown instead of duration is low")
		print (MODULE .. ": demo - Enable/Disable suggesting Demoralizing Shout")
		print (MODULE .. ": shout - Enable/Disable suggesting Commanding/Battle Shout")
		return
	end

	local Cmd, Args = strsplit(" ", cmd:lower(), 2)

	if Cmd == "demo" then
		if options.demo == true then
			options.demo = false
			print (MODULE .. ": Disabled suggesting Demoralizing Shout")
		else
			options.demo = true
			print (MODULE .. ": Enabled suggesting Demoralizing Shout will show")
		end
	elseif Cmd == "tclap" then
		if options.tclap == true then
			options.tclap = false
			print (MODULE .. ": Disabled suggesting Thunder Clap when its off cooldown instead of duration is low")
		else
			options.tclap = true
			print (MODULE .. ": Enabled suggesting Thunder Clap when its off cooldown instead of duration is low")
		end
	elseif Cmd == "shout" then
		if options.shout == true then
			options.shout = false
			print (MODULE .. ": Disabled suggesting Commanding/Battle Shout")
		else
			options.shout = true
			print (MODULE .. ": Enabled suggesting Commanding/Battle Shout")
		end
	else
		print (MODULE .. ": Options:")
		print (MODULE .. ": tclap - Enable/Disable suggesting Thunder Clap when its off cooldown instead of duration is low")
		print (MODULE .. ": demo - Enable/Disable suggesting Demoralizing Shout")
		print (MODULE .. ": shout - Enable/Disable suggesting Commanding/Battle Shout")
	end
end


Faceroller:RegisterModule("Prot warrior", Init, NextShot, Options)